@@ -1,6 +1,8 @@
 from stftpitchshift.cepster import lifter
+from stftpitchshift.normalizer import normalize
 from stftpitchshift.pitcher import shiftpitch
-from stftpitchshift.stft import stft, istft, spectrogram
+from stftpitchshift.resampler import linear as resample
+from stftpitchshift.stft import stft, istft
 from stftpitchshift.vocoder import encode, decode
 
 import numpy as np
@@ -22,11 +24,13 @@ class StftPitchShift:
         self.hopsize = hopsize
         self.samplerate = samplerate
 
-    def shiftpitch(self, input, factors = 1, quefrency = 0):
+    def shiftpitch(self, input, factors = 1, quefrency = 0, distortion = 1, normalization = False):
         '''
         :param input: The input signal.
         :param factors: The fractional pitch shifting factors.
         :param quefrency: The optional formant lifter quefrency in seconds.
+        :param distortion: The fractional timbre shifting factor.
+        :param normalization Optionally enable spectral rms normalization.
         :return: The output signal of the equal size.
         '''
 
@@ -43,11 +47,14 @@ class StftPitchShift:
         quefrency = int(quefrency * samplerate)
 
         frames = stft(input, framesize, hopsize)
+        frames = encode(frames, framesize, hopsize, samplerate)
+
+        if normalization:
+
+            frames0 = frames.copy()
 
         if quefrency:
 
-            frames = encode(frames, framesize, hopsize, samplerate)
-
             envelopes = lifter(frames, quefrency)
 
             mask = isnotnormal(envelopes)
@@ -55,21 +62,32 @@ class StftPitchShift:
             frames.real /= envelopes
             frames.real[mask] = 0
 
-            frames = shiftpitch(frames, factors)
+            if distortion != 1:
+
+                envelopes[mask] = 0
+
+                for i in range(len(envelopes)):
+
+                    envelopes[i] = resample(envelopes[i], distortion)
+
+                mask = isnotnormal(envelopes)
+
+            frames = shiftpitch(frames, factors, samplerate)
 
             frames.real *= envelopes
             frames.real[mask] = 0
 
-            frames = decode(frames, framesize, hopsize, samplerate)
-
         else:
 
-            frames = encode(frames, framesize, hopsize, samplerate)
-            frames = shiftpitch(frames, factors)
-            frames = decode(frames, framesize, hopsize, samplerate)
+            frames = shiftpitch(frames, factors, samplerate)
+
+        if normalization:
+
+            frames = normalize(frames, frames0)
+
+        frames = decode(frames, framesize, hopsize, samplerate)
 
         output = istft(frames, framesize, hopsize)
-
         output.resize(np.shape(input))
 
         return output
